use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Alders
Alders'
Anwar
Atria
Boris
Cyrill
Däppen
Heumann
Joaquín
José
Manni
Mohammad
Novgorodcev
Olaf
Open
This
bdaeppen
cynovg
github
jjatria
lib
libs
mohammad
olaf
ot
script
